@@ -985,6 +985,7 @@ static int get_vulkan_api_version(struct pci_dev *dev, char *vulkan_version, boo
 	int err = 0;
 #if HAS_Vulkan
 	uint32_t i, device_count = 0;
+	bool gpu_found = false;
 	bool use_device_id = false;
 	VkInstance instance = {0};
 	VkPhysicalDevice *devices = NULL;
@@ -1090,22 +1091,19 @@ static int get_vulkan_api_version(struct pci_dev *dev, char *vulkan_version, boo
 		.pEnabledFeatures = NULL,
 	};
 # endif /* VK_KHR_ACCELERATION_STRUCTURE_EXTENSION_NAME */
-	VkPhysicalDevicePCIBusInfoPropertiesEXT bus_info =
-	{
-		.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PCI_BUS_INFO_PROPERTIES_EXT,
-		.pNext = NULL,
-	};
-	VkPhysicalDeviceProperties2 prop2 =
-	{
-		.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2,
-		.pNext = &bus_info,
-	};
+	VkPhysicalDevicePCIBusInfoPropertiesEXT bus_info = {0};
+	bus_info.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PCI_BUS_INFO_PROPERTIES_EXT;
 
-	for(i = 0; i < device_count; i++)
+	VkPhysicalDeviceProperties2 prop2 = {0};
+	prop2.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2;
+	prop2.pNext = &bus_info;
+
+	for(i = 0; (i < device_count) && !gpu_found; i++)
 	{
-		VkDevice vk_dev = {0};
+		MSG_DEBUG("Looping into Vulkan device %lu", i);
 # ifdef VK_EXT_PCI_BUS_INFO_EXTENSION_NAME
-		if((err = vkCreateDevice(devices[i], &check_pci_bus_info, NULL, &vk_dev)) != VK_SUCCESS)
+		VkDevice vk_dev_bus_info = {0};
+		if((err = vkCreateDevice(devices[i], &check_pci_bus_info, NULL, &vk_dev_bus_info)) != VK_SUCCESS)
 		{
 			MSG_WARNING(_("Failed to create Vulkan for device %u (%s)"), i, string_VkResult(err));
 
@@ -1115,37 +1113,47 @@ static int get_vulkan_api_version(struct pci_dev *dev, char *vulkan_version, boo
 				use_device_id = true;
 			}
 		}
+		vkDestroyDevice(vk_dev_bus_info, NULL);
 # else
         use_device_id = true;
 # endif /* VK_EXT_PCI_BUS_INFO_EXTENSION_NAME */
-
 		vkGetPhysicalDeviceProperties2(devices[i], &prop2);
-		if(((uint32_t)dev->domain    == bus_info.pciDomain    &&
-		    dev->bus                 == bus_info.pciBus       &&
-		    dev->dev                 == bus_info.pciDevice    &&
-		    dev->func                == bus_info.pciFunction) ||
-		   (use_device_id &&
-		    (uint32_t)dev->device_id == prop2.properties.deviceID))
+		if(use_device_id && ((uint32_t) dev->device_id != prop2.properties.deviceID))
 		{
+			MSG_DEBUG("Vulkan device %lu: use only deviceID but device %u does not match device %lu", i, dev->device_id, prop2.properties.deviceID);
+			continue;
+		}
+		else if(!use_device_id && ((uint32_t) dev->domain   != bus_info.pciDomain    ||
+		                           dev->bus                 != bus_info.pciBus       ||
+		                           dev->dev                 != bus_info.pciDevice    ||
+		                           dev->func                != bus_info.pciFunction))
+		{
+			MSG_DEBUG("Vulkan device %lu: device does not match with VkPhysicalDevicePCIBusInfoPropertiesEXT", i);
+			continue;
+		}
+		else
+			MSG_DEBUG("Vulkan device %lu: device matches with pci_dev", i);
+
 # ifdef VK_KHR_ACCELERATION_STRUCTURE_EXTENSION_NAME
-			if (vkCreateDevice(devices[i], &check_rt, NULL, &vk_dev) == VK_SUCCESS)
-				*vulkan_rt = true;
+		VkDevice vk_dev_rt = {0};
+		*vulkan_rt = (vkCreateDevice(devices[i], &check_rt, NULL, &vk_dev_rt) == VK_SUCCESS);
+		MSG_DEBUG("Vulkan device %lu: Ray Tracing support is %s", i, *vulkan_rt ? "ON" : "OFF");
+		vkDestroyDevice(vk_dev_rt, NULL);
 # endif /* VK_KHR_ACCELERATION_STRUCTURE_EXTENSION_NAME */
 
-			snprintf(vulkan_version, MAXSTR, "%d.%d.%d",
+		snprintf(vulkan_version, MAXSTR, "%d.%d.%d",
 # if(VK_API_VERSION_MAJOR && VK_API_VERSION_MINOR && VK_API_VERSION_PATCH)
-				VK_API_VERSION_MAJOR(prop2.properties.apiVersion),
-				VK_API_VERSION_MINOR(prop2.properties.apiVersion),
-				VK_API_VERSION_PATCH(prop2.properties.apiVersion)
+			VK_API_VERSION_MAJOR(prop2.properties.apiVersion),
+			VK_API_VERSION_MINOR(prop2.properties.apiVersion),
+			VK_API_VERSION_PATCH(prop2.properties.apiVersion)
 # else
-				VK_VERSION_MAJOR(prop2.properties.apiVersion),
-				VK_VERSION_MINOR(prop2.properties.apiVersion),
-				VK_VERSION_PATCH(prop2.properties.apiVersion)
+			VK_VERSION_MAJOR(prop2.properties.apiVersion),
+			VK_VERSION_MINOR(prop2.properties.apiVersion),
+			VK_VERSION_PATCH(prop2.properties.apiVersion)
 # endif /* (VK_API_VERSION_MAJOR && VK_API_VERSION_MINOR && VK_API_VERSION_PATCH) */
-			);
-			vkDestroyDevice(vk_dev, NULL);
-			break;
-		}
+		);
+		MSG_DEBUG("Vulkan device %lu: version is '%s'", i, vulkan_version);
+		gpu_found = true;
 	}
 	vkDestroyInstance(instance, NULL);
 	free(devices);
@@ -1164,6 +1172,7 @@ static int get_gpu_comp_unit(struct pci_dev *dev, uint32_t *comp_unit, char *com
 {
 	int ret_cl = 0;
 #if HAS_OpenCL
+	bool gpu_found = false;
 	uint8_t ret_domain_nv = 0, ret_bus_nv = 0, ret_dev_nv = 0;
 	char platform_name[MAXSTR] = "", platform_version[MAXSTR] = "", device_name[MAXSTR] = "", device_version[MAXSTR] = "", cl_ver[MAXSTR] = "";
 	cl_uint num_pf = 0, num_ocl_dev = 0, ocl_vendor = 0, amd_gfx_major = 0;
@@ -1193,14 +1202,16 @@ static int get_gpu_comp_unit(struct pci_dev *dev, uint32_t *comp_unit, char *com
 		return ret_cl;
 	}
 
-	for (i = 0; i < num_pf; i++) // find GPU devices
+	for (i = 0; (i < num_pf) && !gpu_found; i++) // find GPU devices
 	{
+		MSG_DEBUG("Looping into OpenCL platform %u", i);
 		ret_cl = clGetPlatformInfo(platforms[i], CL_PLATFORM_NAME, sizeof(platform_name), platform_name, NULL); // get platform name
 		if (ret_cl != CL_SUCCESS)
 		{
 			MSG_ERROR(_("failed to get name for platform %u (%s)"), i, opencl_error(ret_cl));
 			continue;
 		}
+		MSG_DEBUG("OpenCL platform %u: name is '%s'", i, platform_name);
 
 		ret_cl = clGetPlatformInfo(platforms[i], CL_PLATFORM_VERSION, sizeof(platform_version), platform_version, NULL); // get platform version
 		if (ret_cl != CL_SUCCESS)
@@ -1208,29 +1219,33 @@ static int get_gpu_comp_unit(struct pci_dev *dev, uint32_t *comp_unit, char *com
 			MSG_ERROR(_("failed to get version for platform %u (%s)"), i, opencl_error(ret_cl));
 			continue;
 		}
+		MSG_DEBUG("OpenCL platform %u: version is '%s'", i, platform_version);
 
+		num_ocl_dev = 0;
 		ret_cl = clGetDeviceIDs(platforms[i], CL_DEVICE_TYPE_ALL, 0, NULL, &num_ocl_dev); // get number of device
-		if (ret_cl != CL_SUCCESS)
+		if((ret_cl != CL_SUCCESS) || (num_ocl_dev == 0))
 		{
-			MSG_ERROR(_("failed to find number of OpenCL devices for platform '%s %s' (%s)"), platform_name, platform_version, opencl_error(ret_cl));
+			MSG_ERROR(_("failed to find number of OpenCL devices for platform '%s %s' (%s)"), platform_name, platform_version, (num_ocl_dev == 0) ? _("0 device") : opencl_error(ret_cl));
 			continue;
 		}
+		MSG_DEBUG("OpenCL platform %u: found %u devices", i, num_ocl_dev);
 
 		devices = (cl_device_id*) malloc(sizeof(cl_device_id) * num_ocl_dev);
 		ALLOC_CHECK(devices);
 		ret_cl = clGetDeviceIDs(platforms[i], CL_DEVICE_TYPE_ALL, num_ocl_dev, devices, NULL); // get all devices
-		if (ret_cl != CL_SUCCESS)
+		if(ret_cl != CL_SUCCESS)
 		{
 			MSG_ERROR(_("failed to get all of OpenCL devices for platform '%s %s' (%s)"), platform_name, platform_version, opencl_error(ret_cl));
 			continue;
 		}
-		MSG_DEBUG("Number of OpenCL devices for platform '%s %s': %u", platform_name, platform_version, num_ocl_dev);
 
-		for (j = 0; j < num_ocl_dev; j++)
+		for (j = 0; (j < num_ocl_dev) && !gpu_found; j++)
 		{
+			MSG_DEBUG("Looping into OpenCL platform %u, device %u", i, j);
 			CLINFO(devices[j], CL_DEVICE_VENDOR_ID, ocl_vendor);
 			if (dev->vendor_id != ocl_vendor)
 				continue;
+			MSG_DEBUG("OpenCL platform %u, device %u: found vendor 0x%X", i, j, ocl_vendor);
 
 			ret_cl = clGetDeviceInfo(devices[j], CL_DEVICE_NAME, sizeof(device_name), device_name, NULL);
 			if (ret_cl != CL_SUCCESS)
@@ -1238,6 +1253,7 @@ static int get_gpu_comp_unit(struct pci_dev *dev, uint32_t *comp_unit, char *com
 				MSG_ERROR(_("failed to get name for device %u (%s)"), j, opencl_error(ret_cl));
 				continue;
 			}
+			MSG_DEBUG("OpenCL platform %u, device %u: name is '%s'", i, j, device_name);
 
 			ret_cl = clGetDeviceInfo(devices[j], CL_DEVICE_VERSION, sizeof(device_version), device_version, NULL);
 			if (ret_cl != CL_SUCCESS || string_is_empty(device_version))
@@ -1245,14 +1261,16 @@ static int get_gpu_comp_unit(struct pci_dev *dev, uint32_t *comp_unit, char *com
 				MSG_ERROR(_("failed to get version for device %u (%s)"), j, opencl_error(ret_cl));
 				continue;
 			}
+			MSG_DEBUG("OpenCL platform %u, device %u: version is '%s'", i, j, device_version);
+
 			size_t prefix_len = strlen("OpenCL ");
 			strncpy(cl_ver, &device_version[prefix_len], MAXSTR - prefix_len);
 			snprintf(cl_version, MAXSTR, "%s", cl_ver);
 
-			MSG_DEBUG("OpenCL device %u is '%s %s'", j, device_name, device_version);
 			switch (ocl_vendor)
 			{
 				case DEV_VENDOR_ID_AMD:
+					MSG_DEBUG("OpenCL platform %u, device %u: vendor is AMD", i, j);
 					ret_cl = CLINFO(devices[j], CL_DEVICE_TOPOLOGY_AMD, topo_amd);
 					if (ret_cl != CL_SUCCESS)
 					{
@@ -1270,6 +1288,7 @@ static int get_gpu_comp_unit(struct pci_dev *dev, uint32_t *comp_unit, char *com
 							MSG_WARNING(_("OpenCL driver for '%s %s' does not support CL_DEVICE_GFXIP_MAJOR_AMD (%s)"), device_name, device_version, opencl_error(ret_cl));
 							amd_gfx_major = 0;
 						}
+						MSG_DEBUG("OpenCL platform %u, device %u: CL_DEVICE_GFXIP_MAJOR_AMD is %u", i, j, amd_gfx_major);
 
 						if (10 <= amd_gfx_major) // for RDNA, gfx10+
 							snprintf(comp_unit_type, MAXSTR, "%s", "WGP"); // Workgroup Processor
@@ -1283,9 +1302,12 @@ static int get_gpu_comp_unit(struct pci_dev *dev, uint32_t *comp_unit, char *com
 							*comp_unit = 0;
 							continue;
 						}
+						MSG_DEBUG("OpenCL platform %u, device %u: found %lu %s", i, j, *comp_unit, comp_unit_type);
+						gpu_found = true;
 					}
 					break;
 				case DEV_VENDOR_ID_INTEL:
+					MSG_DEBUG("OpenCL platform %u, device %u: vendor is Intel", i, j);
 					ret_cl = CLINFO(devices[j], CL_DEVICE_MAX_COMPUTE_UNITS, *comp_unit);
 					if (ret_cl != CL_SUCCESS)
 					{
@@ -1294,8 +1316,11 @@ static int get_gpu_comp_unit(struct pci_dev *dev, uint32_t *comp_unit, char *com
 						continue;
 					}
 					snprintf(comp_unit_type, MAXSTR, "%s", "EU"); // Execution Unit
+					MSG_DEBUG("OpenCL platform %u, device %u: found %lu %s", i, j, *comp_unit, comp_unit_type);
+					gpu_found = true;
 					break;
 				case DEV_VENDOR_ID_NVIDIA:
+					MSG_DEBUG("OpenCL platform %u, device %u: vendor is NVIDIA", i, j);
 					ret_domain_nv = CLINFO(devices[j], CL_DEVICE_PCI_DOMAIN_ID_NV, ocl_domain_nv);
 					ret_bus_nv    = CLINFO(devices[j], CL_DEVICE_PCI_BUS_ID_NV,    ocl_bus_nv);
 					ret_dev_nv    = CLINFO(devices[j], CL_DEVICE_PCI_SLOT_ID_NV,   ocl_dev_nv); // Slot == Device
@@ -1319,6 +1344,8 @@ static int get_gpu_comp_unit(struct pci_dev *dev, uint32_t *comp_unit, char *com
 							continue;
 						}
 						snprintf(comp_unit_type, MAXSTR, "%s", "SM"); // Streaming Multiprocessor
+						MSG_DEBUG("OpenCL platform %u, device %u: found %lu %s", i, j, *comp_unit, comp_unit_type);
+						gpu_found = true;
 					}
 					break;
 				default:
@@ -1420,10 +1447,10 @@ static int find_devices(Labels *data)
 			casprintf(&data->tab_graphics[VALUE][GPU1MODEL  + data->gpu_count * GPUFIELDS], false, "%s", DEVICE_PRODUCT_STR(dev));
 			casprintf(&data->tab_graphics[VALUE][GPU1DIDRID + data->gpu_count * GPUFIELDS], false, "0x%04X:0x%02X", dev->device_id, pci_read_byte(dev, PCI_REVISION_ID));
 			casprintf(&data->tab_graphics[VALUE][GPU1CU     + data->gpu_count * GPUFIELDS], true,  "%d %s", comp_unit, comp_unit_type);
-			casprintf(&data->tab_graphics[VALUE][GPU1VKRT   + data->gpu_count * GPUFIELDS], true,  "%s", vulkan_ray_tracing ? _("Enabled") : _("Disabled"));
-			casprintf(&data->tab_graphics[VALUE][GPU1GLVER  + data->gpu_count * GPUFIELDS], true,  "%s", gl_ver);
-			casprintf(&data->tab_graphics[VALUE][GPU1VKVER  + data->gpu_count * GPUFIELDS], true,  "%s", vk_ver);
-			casprintf(&data->tab_graphics[VALUE][GPU1CLVER  + data->gpu_count * GPUFIELDS], true,  "%s", cl_ver);
+			casprintf(&data->tab_graphics[VALUE][GPU1VKRT   + data->gpu_count * GPUFIELDS], false, "%s", vulkan_ray_tracing ? _("Enabled") : _("Disabled"));
+			casprintf(&data->tab_graphics[VALUE][GPU1GLVER  + data->gpu_count * GPUFIELDS], false, "%s", gl_ver);
+			casprintf(&data->tab_graphics[VALUE][GPU1VKVER  + data->gpu_count * GPUFIELDS], false, "%s", vk_ver);
+			casprintf(&data->tab_graphics[VALUE][GPU1CLVER  + data->gpu_count * GPUFIELDS], false, "%s", cl_ver);
 			data->gpu_count++;
 		}
 	}
