@@ -42,6 +42,10 @@ static int call_libcpuid_static(Labels *data);
 static int call_libcpuid_dynamic(Labels *data);
 /* Required: HAS_LIBCPUID */
 
+/* MSRs raw values provided by libcpuid */
+static int call_libcpuid_msr_debug(Labels *data, uint16_t all_cpu_count);
+/* Required: HAS_LIBCPUID && DAEMON_UP */
+
 /* MSRs static values provided by libcpuid */
 static int call_libcpuid_msr_static(Labels *data);
 /* Required: HAS_LIBCPUID && DAEMON_UP */
@@ -89,7 +93,7 @@ static int system_static(Labels *data);
 #if (HAS_LIBSYSTEM)
 /* Dynamic elements for System tab, provided by libprocps/libstatgrab */
 static int system_dynamic(Labels *data);
-/* Required: HAS_LIBPROCPS || HAS_LIBSTATGRAB */
+/* Required: HAS_LIBPROC2 || HAS_LIBPROCPS || HAS_LIBSTATGRAB */
 #endif /* HAS_LIBSYSTEM */
 
 /* Report score of benchmarks */
