@@ -31,6 +31,7 @@
 #include "cpu-x.h"
 #include "tui_ncurses.h"
 
+static unsigned tab_memory_scroll = 0;
 static void (*func_ptr[])(WINDOW*, const SizeInfo, Labels*) =
 {
 	ntab_cpu,
@@ -65,7 +66,7 @@ static const Colors color[] =
 void start_tui_ncurses(Labels *data)
 {
 	int err = 0, ch = 0;
-	const SizeInfo info = { .height = LINE_COUNT, .width = 70, .start = 1, .tb = 2, .tm = 26, .te = 48 };
+	const SizeInfo info = { .height = LINE_COUNT, .width = 76, .start = 1, .tb = 2, .tm = 26, .te = 48 };
 	NThrd refr = { .data = data, .info = info };
 	WINDOW *win;
 
@@ -122,6 +123,7 @@ void start_tui_ncurses(Labels *data)
 		switch(ch)
 		{
 			case KEY_LEFT:
+			case SHIFT_TAB:
 				/* Switch to left tab */
 				if(opts->selected_page > NO_CPU)
 				{
@@ -134,6 +136,7 @@ void start_tui_ncurses(Labels *data)
 				}
 				break;
 			case KEY_RIGHT:
+			case KEY_TAB:
 				/* Switch to right tab */
 				if(opts->selected_page < NO_ABOUT)
 				{
@@ -149,7 +152,8 @@ void start_tui_ncurses(Labels *data)
 				if(opts->selected_page == NO_CPU && opts->selected_core > 0)
 				{
 					opts->selected_core--;
-					print_activecore(win);
+					change_current_core_id(data);
+					print_activecore(win, data);
 				}
 				else if(opts->selected_page == NO_CACHES && opts->bw_test > 0)
 				{
@@ -168,17 +172,33 @@ void start_tui_ncurses(Labels *data)
 				}
 				break;
 			case KEY_NPAGE:
-				if(opts->selected_page == NO_BENCH && data->b_data->threads > 1 && !data->b_data->run)
+				if(opts->selected_page == NO_CPU && (int) opts->selected_type > 0)
+				{
+					opts->selected_type--;
+					data->l_data->change_type = true;
+					change_current_core_id(data);
+					print_activetype(win);
+				}
+				else if(opts->selected_page == NO_BENCH && data->b_data->threads > 1 && !data->b_data->run)
 				{
 					data->b_data->threads--;
 					print_paramthreads(win, info, data);
 				}
 				break;
+			case KEY_HOME:
+				if(opts->selected_page == NO_MEMORY && tab_memory_scroll > 0)
+				{
+					tab_memory_scroll--;
+					main_win(win, info, data);
+					ntab_memory(win, info, data);
+				}
+				break;
 			case KEY_UP:
-				if(opts->selected_page == NO_CPU && (int) opts->selected_core < data->cpu_count - 1)
+				if(opts->selected_page == NO_CPU && (int) opts->selected_core < data->current_cpu_count - 1)
 				{
 					opts->selected_core++;
-					print_activecore(win);
+					change_current_core_id(data);
+					print_activecore(win, data);
 				}
 				else if(opts->selected_page == NO_CACHES && (int) opts->bw_test < data->w_data->test_count - 1)
 				{
@@ -197,12 +217,27 @@ void start_tui_ncurses(Labels *data)
 				}
 				break;
 			case KEY_PPAGE:
-				if(opts->selected_page == NO_BENCH && data->b_data->threads < data->cpu_count && !data->b_data->run)
+				if(opts->selected_page == NO_CPU && (int) opts->selected_type < data->type_count - 1)
+				{
+					opts->selected_type++;
+					data->l_data->change_type = true;
+					change_current_core_id(data);
+					print_activetype(win);
+				}
+				else if(opts->selected_page == NO_BENCH && data->b_data->threads < data->all_cpu_count && !data->b_data->run)
 				{
 					data->b_data->threads++;
 					print_paramthreads(win, info, data);
 				}
 				break;
+			case KEY_END:
+				if(opts->selected_page == NO_MEMORY && tab_memory_scroll + DISPLAYABLE_DIMM_COUNT < data->dimm_count)
+				{
+					tab_memory_scroll++;
+					main_win(win, info, data);
+					ntab_memory(win, info, data);
+				}
+				break;
 			case 'f':
 				if(opts->selected_page == NO_BENCH && !data->b_data->run)
 				{
@@ -259,15 +294,6 @@ void start_tui_ncurses(Labels *data)
 
 /************************* Private functions *************************/
 
-/* Convert keys when an alternative mapping is used */
-#define ALT_CODE 27
-#define ALT(x)   (x & ALT_CODE)
-
-/* glibc's term.h pulls in sys/ttydefaults.h which has it, but musl's does not. */
-#ifndef CTRL
-#define CTRL(x)	(x&037)
-#endif
-
 static int convert_char(int ch)
 {
 	int i = 0;
@@ -437,24 +463,22 @@ static void nrefresh(NThrd *refr)
 			break;
 		case NO_GRAPHICS:
 			do_refresh(data);
-			line = LINE_3;
+			line = LINE_6;
 			for(i = 0; i < data->gpu_count * GPUFIELDS; i += GPUFIELDS)
 			{
-				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1DIDRID      + i], data->tab_graphics[VALUE][GPU1DIDRID     + i]);
-				line++;
-				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1PCIE        + i], data->tab_graphics[VALUE][GPU1PCIE       + i]);
+				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1PCIE        + i], data->tab_graphics[VALUE][GPU1PCIE        + i]);
 				line++;
 				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1TEMPERATURE + i], data->tab_graphics[VALUE][GPU1TEMPERATURE + i]);
 				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1USAGE       + i], data->tab_graphics[VALUE][GPU1USAGE       + i]);
 				line++;
-				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1MEMUSED     + i], data->tab_graphics[VALUE][GPU1MEMUSED     + i]);
+				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1VOLTAGE     + i], data->tab_graphics[VALUE][GPU1VOLTAGE     + i]);
+				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1POWERAVG    + i], data->tab_graphics[VALUE][GPU1POWERAVG    + i]);
 				line++;
-				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1CORECLOCK + i], data->tab_graphics[VALUE][GPU1CORECLOCK     + i]);
-				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1MEMCLOCK  + i], data->tab_graphics[VALUE][GPU1MEMCLOCK      + i]);
+				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1CORECLOCK   + i], data->tab_graphics[VALUE][GPU1CORECLOCK   + i]);
+				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1MEMCLOCK    + i], data->tab_graphics[VALUE][GPU1MEMCLOCK    + i]);
 				line++;
-				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1VOLTAGE   + i], data->tab_graphics[VALUE][GPU1VOLTAGE       + i]);
-				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1POWERAVG  + i], data->tab_graphics[VALUE][GPU1POWERAVG      + i]);
-				line += LINE_3 - 1;
+				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1MEMUSED     + i], data->tab_graphics[VALUE][GPU1MEMUSED     + i]);
+				line += LINE_6 - 1;
 			}
 			break;
 		case NO_BENCH:
@@ -481,18 +505,22 @@ static void print_help()
 	printw("%s\n", _("This help describes how to use this Text-based User Interface."));
 
 	printw("\n%s\n", _("Global keys:"));
-	printw("\t%s\n", _("Press 'left' key to switch in left tab."));
-	printw("\t%s\n", _("Press 'right' key to switch in right tab."));
+	printw("\t%s\n", _("Press 'left' key or 'tab' key to switch in left tab."));
+	printw("\t%s\n", _("Press 'right' key or 'shift + tab' keys to switch in right tab."));
+	printw("\t%s\n", _("Press 'home' key to scroll up in a page."));
+	printw("\t%s\n", _("Press 'end' key to scroll down in a page."));
 	printw("\t%s\n", _("Press 'h' key to see this help."));
 	printw("\t%s\n", _("Press 'q' key to exit."));
 
 	printw("\n%s\n", _("CPU tab:"));
-	printw("\t%s\n", _("Press 'down' key to decrease core number to monitor."));
-	printw("\t%s\n", _("Press 'up' key to increase core number to monitor."));
+	printw("\t%s\n", _("Press 'next page' key to decrease core type number to monitor."));
+	printw("\t%s\n", _("Press 'previous page' key to increase core type number to monitor."));
+	printw("\t%s\n", _("Press 'down' key to decrease CPU core number to monitor."));
+	printw("\t%s\n", _("Press 'up' key to increase CPU core number to monitor."));
 
 	printw("\n%s\n", _("Caches tab:"));
 	printw("\t%s\n", _("Press 'down' key to switch to previous test."));
-	printw("\t%s\n", _("Press 'up' key' to switch to next test."));
+	printw("\t%s\n", _("Press 'up' key to switch to next test."));
 
 	printw("\n%s\n", _("Bench tab:"));
 	printw("\t%s\n", _("Press 'down' key to decrement benchmark duration."));
@@ -528,7 +556,7 @@ static void main_win(WINDOW *win, const SizeInfo info, Labels *data)
 	box(win, 0, 0);
 
 	mvwprintwc(win, TITLE_LINE, info.width / 2 - strlen(PRGNAME) / 2, TITLE_COLOR, PRGNAME);
-	mvwprintwc(win, HEADER_LINE, 2, DEFAULT_COLOR, PRGNAME);
+	mvwprintwc(win, HEADER_LINE, 12, DEFAULT_COLOR, PRGNAME);
 	mvwprintwc(win, HEADER_LINE, info.width / 2, DEFAULT_COLOR, data->tab_about[VERSIONSTR]);
 
 	for(i = 1; i < info.width - 1; i++)
@@ -548,16 +576,31 @@ static void main_win(WINDOW *win, const SizeInfo info, Labels *data)
 	wrefresh(win);
 }
 
-/* Display active Core in CPU tab */
-#define BUFF_SIZE 16
-static void print_activecore(WINDOW *win)
+/* Display active Type in CPU tab */
+#define BUFF_SIZE 32
+static void print_activetype(WINDOW *win)
 {
 	char buff[BUFF_SIZE];
-	snprintf(buff, BUFF_SIZE, _("Core #%i"), opts->selected_core);
+	snprintf(buff, BUFF_SIZE, _("Type #%i"), opts->selected_type);
 	mvwprintwc(win, LINE_18, 4, DEFAULT_COLOR, buff);
 }
 #undef BUFF_SIZE
 
+/* Display active Core in CPU tab */
+#define BUFF_SIZE 16
+static void print_activecore(WINDOW *win, Labels *data)
+{
+	char buff[BUFF_SIZE];
+#if HAS_LIBCPUID
+	snprintf(buff, BUFF_SIZE, "%s #%i", get_core_type_name(data->l_data->system_id.cpu_types[opts->selected_type].purpose), opts->selected_core);
+#else
+	UNUSED(data);
+	snprintf(buff, BUFF_SIZE, _("Core #%i"), opts->selected_core);
+#endif /* HAS_LIBCPUID */
+	mvwprintwc(win, LINE_18, 18, DEFAULT_COLOR, buff);
+}
+#undef BUFF_SIZE
+
 /* CPU tab */
 static void ntab_cpu(WINDOW *win, const SizeInfo info, Labels *data)
 {
@@ -612,13 +655,13 @@ static void ntab_cpu(WINDOW *win, const SizeInfo info, Labels *data)
 	/* Cache frame */
 	frame(win, LINE_11, middle, LINE_16, info.width - 1, data->objects[FRAMCACHE]);
 	line = LINE_12;
-	for(i = LEVEL1D; i < SOCKETS; i++)
+	for(i = LEVEL1D; i < CORES; i++)
 		mvwprintw2c(win, line++, middle + 1, "%12s: %s", data->tab_cpu[NAME][i], data->tab_cpu[VALUE][i]);
 
 	/* Last frame */
 	frame(win, LINE_17, info.start, LINE_19, info.width - 1, "");
-	print_activecore(win);
-	mvwprintw2c(win, LINE_18, 18,  "%s: %2s", data->tab_cpu[NAME][SOCKETS], data->tab_cpu[VALUE][SOCKETS]);
+	print_activetype(win);
+	print_activecore(win, data);
 	mvwprintw2c(win, LINE_18, 36, "%s: %2s", data->tab_cpu[NAME][CORES],   data->tab_cpu[VALUE][CORES]);
 	mvwprintw2c(win, LINE_18, 54, "%s: %2s", data->tab_cpu[NAME][THREADS], data->tab_cpu[VALUE][THREADS]);
 
@@ -698,18 +741,24 @@ static void ntab_motherboard(WINDOW *win, const SizeInfo info, Labels *data)
 /* Memory tab */
 static void ntab_memory(WINDOW *win, const SizeInfo info, Labels *data)
 {
-	int i, line = LINE_0;
+	unsigned i, line = LINE_0;
+	const char *delim = ",";
+	char *dimm_line, *dimm_line1, *dimm_line2;
 
 	if(!data->dimm_count)
 		return;
 
 	/* Banks frame */
-	for(i = BANK0; i < data->dimm_count; i++)
+	for(i = tab_memory_scroll; (i < tab_memory_scroll + DISPLAYABLE_DIMM_COUNT) && (i < data->dimm_count); i++)
 	{
-		frame(win, line, info.start, line + 2, info.width - 1, data->objects[FRAMBANK0 + i]);
-		line++;
-		mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_memory[NAME][i], data->tab_memory[VALUE][i]);
+		frame(win, line, info.start, line + 3, info.width - 1, data->objects[FRAMBANK0 + i]);
+		dimm_line  = strdup(data->tab_memory[VALUE][i]);
+		dimm_line1 = strsep(&dimm_line, delim);
+		dimm_line2 = strsep(&dimm_line, delim);
+		mvwprintw2c(win, ++line, info.tb, "%13s: %s", data->tab_memory[NAME][i], dimm_line1);
+		mvwprintwc (win, ++line, info.tb + 14, LABEL_VALUE_COLOR, (dimm_line2 == NULL) ? "" : dimm_line2);
 		line += 2;
+		free(dimm_line);
 	}
 
 	wrefresh(win);
@@ -783,14 +832,14 @@ static void print_activecard(WINDOW *win, const SizeInfo info, Labels *data)
 	if(data->gpu_count == 0)
 		return;
 
-	mvwprintwc(win, LINE_13, info.tb + 1, DEFAULT_COLOR, "#%i: %s", opts->selected_gpu, data->tab_graphics[VALUE][GPU1MODEL + opts->selected_gpu * GPUFIELDS]);
+	mvwprintwc(win, LINE_16, info.tb + 1, DEFAULT_COLOR, "#%i: %s", opts->selected_gpu, data->tab_graphics[VALUE][GPU1MODEL + opts->selected_gpu * GPUFIELDS]);
 	wrefresh(win);
 }
 
 /* Graphics tab */
 static void ntab_graphics(WINDOW *win, const SizeInfo info, Labels *data)
 {
-	int i, j, line, start = LINE_0, end = LINE_11;
+	int i, j, line, start = LINE_0, end = LINE_14;
 
 	if(data->gpu_count == 0)
 		return;
@@ -807,6 +856,8 @@ static void ntab_graphics(WINDOW *win, const SizeInfo info, Labels *data)
 			case GPU1MEMCLOCK:
 			case GPU1POWERAVG:
 			case GPU1DIDRID:
+			case GPU1VKRT:
+			case GPU1VKVER:
 				mvwprintw2c(win, line - 1, info.tm, "%18s: %s", data->tab_graphics[NAME][i], data->tab_graphics[VALUE][i]);
 				break;
 			default:
